// pvcamj.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "pvcamj_128.h"

#define IDC_DIALOG_STATUS		200
#define PI 3.14159265

// Global Variables:
int params[13];
int aqmode,oldx,oldy,oldxmin,oldymin,zoom,pframes,psec;
int16 hCam;
char dumstr[500];
HWND hdlgwnd;
bool running,autoscale,inroi,paintacq,paintprev,paused_acq,paused,writefile;
char szFileName[MAX_PATH];
unsigned short prevdata[128*128];
unsigned short prevdata2[128*128];
double minint,maxint,minaqint,maxaqint,avgaqint;
HDC prevdc,acqdc;
HBITMAP prevbmp,acqbmp;

// Forward declarations of functions included in this code module:
BOOL CALLBACK		DlgProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK CamDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam);
int round(float x);
int SetDlgItemFlt(float x, HWND hWnd, int ITEM_ID);
float GetDlgItemFlt(HWND hWnd, int ITEM_ID);
int set_params();
void AcquireContinuous(void* ch);
void PreviewContinuous(void* ch);
void update_preview_image(void *dummy);
void update_acquire_image(void *dummy);
int minval(unsigned short* data,int length);
int maxval(unsigned short* data,int length);
double avgval(unsigned short* data,int length);

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	return DialogBox(hInstance, MAKEINTRESOURCE(IDD_MAIN), NULL, DlgProc);
}

// Message handler for about box.
BOOL CALLBACK DlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc,bmpdc;
	HBITMAP hBmp;
	PAINTSTRUCT ps;
	int statwidths[] = {-1};
	INITCOMMONCONTROLSEX ccs;
	char cam_name[CAM_NAME_LEN];
	rs_bool camera_status;
	int i,j;
	int dumint,size,sizex,sizey;
	double dumdouble;
	
	switch(Message)
	{
		case WM_INITDIALOG:
			{
			hdlgwnd=hwnd;
			running=false;
			HDC hdc=GetDC(hwnd);
			prevdc=CreateCompatibleDC(hdc);
			acqdc=CreateCompatibleDC(hdc);
			prevbmp=CreateCompatibleBitmap(hdc,128,128);
			SelectObject(prevdc,prevbmp);
			acqbmp=CreateCompatibleBitmap(hdc,128,128);
			SelectObject(acqdc,acqbmp);
			ReleaseDC(hwnd,hdc);
			// This is where we set up the dialog box, and initialise any default values
			// Set up the status window
			ZeroMemory(&ccs, sizeof(ccs));
			ccs.dwICC=ICC_BAR_CLASSES;
			InitCommonControlsEx(&ccs);
			HWND hStatus = CreateWindowEx(0, STATUSCLASSNAME, NULL,
				WS_CHILD | WS_VISIBLE, 0, 0, 0, 0,
				hwnd, (HMENU)IDC_DIALOG_STATUS, GetModuleHandle(NULL), NULL);

			if(hStatus==NULL){MessageBox(hwnd,"status bar is null",NULL,MB_OK);}

			SendMessage(hStatus, SB_SETPARTS, 1, (LPARAM)statwidths);
			
			// Initialize the cursor controls
			minint=0.0;
			SetDlgItemInt(hwnd,IDC_MIN,(int)minint,FALSE);
			maxint=5000.0;
			SetDlgItemInt(hwnd,IDC_MAX,(int)maxint,FALSE);
			minaqint=0.0;
			SetDlgItemInt(hwnd,IDC_MINAQ,(int)minaqint,FALSE);
			maxaqint=5000.0;
			SetDlgItemInt(hwnd,IDC_MAXAQ,(int)maxaqint,FALSE);

			// Initialize the radio buttons
			aqmode=0;
			CheckDlgButton(hwnd,IDC_PREV,BST_CHECKED);
			autoscale=true;
			CheckDlgButton(hwnd,IDC_ASCALE,BST_CHECKED);
			paused_acq=false;
			CheckDlgButton(hwnd,IDC_PAUSE,BST_UNCHECKED);
			pframes=100;
			SetDlgItemInt(hwnd,IDC_PFRAMES,pframes,FALSE);
			psec=60;
			SetDlgItemInt(hwnd,IDC_PSEC,psec,FALSE);
			writefile=true;
			CheckDlgButton(hwnd,IDC_WRITEFILE,BST_CHECKED);
			
	    /* Initialize the PVCam Library and Open the First Camera */
	    pl_pvcam_init();
	    pl_cam_get_name( 0, cam_name );
	    camera_status=pl_cam_open(cam_name, &hCam, OPEN_EXCLUSIVE );
	    if(camera_status){
		    strcpy(dumstr,"Camera ");
				memcpy((void*)(dumstr+7),(void*)cam_name,strlen(cam_name));
				strcat(dumstr," opened for aquisition");
		    SendMessage(hStatus,SB_SETTEXT,0,(LPARAM)dumstr);
		  }
		  else{
		   	MessageBox(hwnd,"Unable to open camera",NULL,MB_OK);
		   	SendMessage(hwnd,WM_CLOSE,0,0);
		   	break;
		  }
	    
	    //set up the default initial parameters
	    params[0]=0; //roi serial start
	    SetDlgItemInt(hwnd,IDC_ROI_X,params[0],FALSE);
	    params[1]=128; //roi serial width
	    SetDlgItemInt(hwnd,IDC_ROI_WIDTH,params[1],FALSE);
	    params[2]=1; //roi serial bin
	    SetDlgItemInt(hwnd,IDC_BINX,params[2],FALSE);
	    params[3]=0; //roi parrallel start
	    SetDlgItemInt(hwnd,IDC_ROI_Y,params[3],FALSE);
	    params[4]=128; //roi parrallel width
	    SetDlgItemInt(hwnd,IDC_ROI_HEIGHT,params[4],FALSE);
	    params[5]=1; //roi parrallel bin
	    SetDlgItemInt(hwnd,IDC_BINY,params[5],FALSE);
	    params[6]=1; //clear cycles
	    params[7]=2; //clearing mode(0-never,1-preexposure,2-presequence...)
	    params[8]=1; //amplifier gain
	    params[9]=0; //on-chip multiplier gain (0-4095)
	    params[10]=1; //pmode(0-normal,1-frame-transfer,...)
	    params[11]=4; //shutter open mode(0-never,1-preexposure,2-presequence,3-pretrigger,4-no change)
	    params[12]=5000; //exposure time (microseconds)
	    SetDlgItemInt(hwnd,IDC_EXP,params[12],FALSE);
	    zoom=1;
	    SetDlgItemInt(hwnd,IDC_ZOOM,zoom,FALSE);
	    
	    if(camera_status){
	    	i=set_params();
		    if(i!=0){
		    	pl_error_message(i,dumstr);
		    	MessageBox(hwnd,dumstr,0,MB_OK);
		   		SendMessage(hwnd,WM_CLOSE,0,0);
				}
			}
			paintacq=true;
			paintprev=true;
			}
		break;
		case WM_SIZE:
			{
			HWND hStatus;
			hStatus = GetDlgItem(hwnd,IDC_DIALOG_STATUS);
			SendMessage(hStatus,WM_SIZE,0,0);
			}
		break;
		case WM_PAINT:
			{
			size=128;
			sizex=params[1]/params[2];
			sizey=params[4]/params[5];
			hdc = BeginPaint(hwnd,&ps);
			if(paintprev){
				for(i=0;i<size;i++){
					for(j=0;j<size;j++){
						dumdouble=((double)prevdata[j+i*size]-minint)/(maxint-minint);
						dumint=(int)(dumdouble*255.0);
						if(dumint>255){dumint=255;}
						if(dumint<0){dumint=0;}
						SetPixel(prevdc,j,i,RGB((BYTE)dumint,(BYTE)dumint,(BYTE)dumint));
					}
				}
				paintprev=false;
			}
			BitBlt(hdc,0,0,size,size,prevdc,0,0,SRCCOPY);
			if(paintacq){
				for(i=0;i<sizex;i++){
					for(j=0;j<sizey;j++){
						dumdouble=((double)prevdata2[j+i*sizex]-minaqint)/(maxaqint-minaqint);
						dumint=(int)(dumdouble*255.0);
						if(dumint>255){dumint=255;}
						if(dumint<0){dumint=0;}
						SetPixel(acqdc,j,i,RGB((BYTE)dumint,(BYTE)dumint,(BYTE)dumint));
					}
				}
				paintacq=false;
			}
			StretchBlt(hdc,400,0,sizex*zoom,sizey*zoom,acqdc,0,0,sizex,sizey,SRCCOPY);
			SelectObject(hdc,GetStockObject(DC_PEN));
			SetDCPenColor(hdc,RGB(0,0,255));
			MoveToEx(hdc,params[0],params[3],NULL);
			LineTo(hdc,params[0]+params[1]-1,params[3]);
			LineTo(hdc,params[0]+params[1]-1,params[3]+params[4]-1);
			LineTo(hdc,params[0],params[3]+params[4]-1);
			LineTo(hdc,params[0],params[3]);
			EndPaint(hwnd, &ps);
			}
		break;
		case WM_LBUTTONDOWN:
		{
			oldx=GET_X_LPARAM(lParam);
			oldy=GET_Y_LPARAM(lParam);
			if(oldx>params[0] && oldx<(params[0]+params[1]-1)){
				if(oldy>params[3] && oldy<(params[3]+params[4]-1)){
					inroi=true;
					oldxmin=params[0];
					oldymin=params[3];
				}
			}
		}
		break;
		case WM_MOUSEMOVE:
		{
			int xPos,yPos;
			if(inroi){
				xPos=GET_X_LPARAM(lParam);
				yPos=GET_Y_LPARAM(lParam);
				params[0]=oldxmin+(xPos-oldx);
				if((params[0]+params[1]-1)>127){params[0]=128-params[1];}
				if(params[0]<0){params[0]=0;}
				params[3]=oldymin+(yPos-oldy);
				if((params[3]+params[4]-1)>127){params[3]=128-params[4];}
				if(params[3]<0){params[3]=0;}
				SetDlgItemInt(hwnd,IDC_ROI_X,params[0],FALSE);
				SetDlgItemInt(hwnd,IDC_ROI_Y,params[3],FALSE);
				InvalidateRect(hwnd,NULL,FALSE);
				UpdateWindow(hwnd);
			}
		}
		break;
		case WM_LBUTTONUP:
		{
			inroi=false;
		}
		break;
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDC_PREV:
				{
					if(!running){
						CheckDlgButton(hwnd,IDC_PREV,BST_CHECKED);
						CheckDlgButton(hwnd,IDC_AQUIRE,BST_UNCHECKED);
						aqmode=0;
					}
				}
				break;
				case IDC_AQUIRE:
				{
					if(!running){
						CheckDlgButton(hwnd,IDC_AQUIRE,BST_CHECKED);
						CheckDlgButton(hwnd,IDC_PREV,BST_UNCHECKED);
						aqmode=1;
					}
				}
				break;
				case IDC_ASCALE:
				{
					if(autoscale){
						autoscale=false;
						CheckDlgButton(hwnd,IDC_ASCALE,BST_UNCHECKED);
					}
					else{
						autoscale=true;
						CheckDlgButton(hwnd,IDC_ASCALE,BST_CHECKED);
					}
				}
				break;
				case IDC_PAUSE:
				{
					if(!running){
						if(paused_acq){
							paused_acq=false;
							CheckDlgButton(hwnd,IDC_PAUSE,BST_UNCHECKED);
						}
						else{
							paused_acq=true;
							CheckDlgButton(hwnd,IDC_PAUSE,BST_CHECKED);
						}
					}
				}
				break;
				case IDC_WRITEFILE:
				{
					if(writefile){
						CheckDlgButton(hwnd,IDC_WRITEFILE,BST_UNCHECKED);
						writefile=false;
					}
					else{
						CheckDlgButton(hwnd,IDC_WRITEFILE,BST_CHECKED);
						writefile=true;
					}
				}
				break;
				case IDC_START:
				{
					if(!running && !paused){
						if(aqmode==0){
							running=true;
							paused=false;
							_beginthread(PreviewContinuous,0,NULL);
						}
						else{
							psec=GetDlgItemInt(hwnd,IDC_PSEC,NULL,FALSE);
							pframes=GetDlgItemInt(hwnd,IDC_PFRAMES,NULL,FALSE);
							params[12]=GetDlgItemInt(hwnd,IDC_EXP,NULL,FALSE);
							for(i=0;i<128*128;i++){
									prevdata2[i]=0;
							}
							OPENFILENAME ofn;
							ZeroMemory(&ofn,sizeof(ofn));
							ofn.lStructSize = sizeof(OPENFILENAME);
							ofn.hwndOwner = hwnd;
							ofn.lpstrFilter = "Binary Files (*.bin)\0*.bin\0";
							ofn.lpstrFile = (LPSTR)szFileName;
							ofn.nMaxFile = MAX_PATH;
							ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
							ofn.lpstrDefExt = "bin";
							if(GetSaveFileName(&ofn)){
								running=true;
								_beginthread(AcquireContinuous,0,NULL);
							}
							else{
								MessageBox(hwnd,"error opening file",NULL,MB_OK);
							}
						}
					}
				}
				break;
				case IDC_STOP:
				{
					running=false;
					paused=false;
				}
				break;
				case ID_CAM_PARM:
				{
					DialogBox(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_CAM_DLG),hwnd,CamDlgProc);
				}
				break;
				case IDOK:
				{
					minint=(double)GetDlgItemInt(hwnd,IDC_MIN,NULL,FALSE);
					maxint=(double)GetDlgItemInt(hwnd,IDC_MAX,NULL,FALSE);
					minaqint=(double)GetDlgItemInt(hwnd,IDC_MINAQ,NULL,FALSE);
					maxaqint=(double)GetDlgItemInt(hwnd,IDC_MAXAQ,NULL,FALSE);
					if(!running){
						params[0]=GetDlgItemInt(hwnd,IDC_ROI_X,NULL,FALSE);
						params[1]=GetDlgItemInt(hwnd,IDC_ROI_WIDTH,NULL,FALSE);
						if(params[1]>128){
							params[1]=128;
							SetDlgItemInt(hwnd,IDC_ROI_WIDTH,params[1],FALSE);
						}
						if(params[0]<0){
							params[0]=0;
							SetDlgItemInt(hwnd,IDC_ROI_X,params[0],FALSE);
						}
						if((params[0]+params[1]-1)>127){
							params[0]=128-params[1];
							SetDlgItemInt(hwnd,IDC_ROI_X,params[0],FALSE);
						}
						params[2]=GetDlgItemInt(hwnd,IDC_BINX,NULL,FALSE);
						params[3]=GetDlgItemInt(hwnd,IDC_ROI_Y,NULL,FALSE);
						params[4]=GetDlgItemInt(hwnd,IDC_ROI_HEIGHT,NULL,FALSE);
						if(params[4]>128){
							params[4]=128;
							SetDlgItemInt(hwnd,IDC_ROI_WIDTH,params[4],FALSE);
						}
						if(params[3]<0){
							params[3]=0;
							SetDlgItemInt(hwnd,IDC_ROI_X,params[3],FALSE);
						}
						if((params[3]+params[4]-1)>127){
							params[3]=128-params[4];
							SetDlgItemInt(hwnd,IDC_ROI_X,params[3],FALSE);
						}
						params[5]=GetDlgItemInt(hwnd,IDC_BINY,NULL,FALSE);
	
						params[12]=GetDlgItemInt(hwnd,IDC_EXP,NULL,FALSE);
						zoom=GetDlgItemInt(hwnd,IDC_ZOOM,NULL,FALSE);
						psec=GetDlgItemInt(hwnd,IDC_PSEC,NULL,FALSE);
						pframes=GetDlgItemInt(hwnd,IDC_PFRAMES,NULL,FALSE);
						paintacq=true;
						InvalidateRect(hwnd,NULL,TRUE);
						UpdateWindow(hwnd);
					}
				}
			}
			break;
		case WM_CLOSE:
			running=false;
			pl_cam_close( hCam );
    	pl_pvcam_uninit();
			DeleteObject(prevbmp);
			DeleteObject(acqbmp);
			DeleteDC(prevdc);
			DeleteDC(acqdc);
			EndDialog(hwnd, 0);
		break;
		default:
			return FALSE;
	}
	return TRUE;
}

BOOL CALLBACK CamDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
		case WM_INITDIALOG:
			{
				//fill the combo boxes with choices and numbers
				HWND hctl;
				int i;
				int16 currtemp;
				float fcurrtemp;
				char temptext[15];
				SetDlgItemInt(hwnd,IDC_CLEAR_CYCLES,params[6],FALSE);
				hctl=GetDlgItem(hwnd,IDC_CLEAR_MODE);
				SendMessage(hctl,CB_RESETCONTENT,0,0);
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"Never");
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"preexposure");
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"presequence");
				SendMessage(hctl,CB_SETCURSEL,(WPARAM)params[7],0);
				hctl=GetDlgItem(hwnd,IDC_GAIN_INDEX);
				SendMessage(hctl,CB_RESETCONTENT,0,0);
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"1");
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"2");
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"3");
				SendMessage(hctl,CB_SETCURSEL,(WPARAM)(params[8]-1),0);
				SetDlgItemInt(hwnd,IDC_GAIN_MULT,params[9],FALSE);
				hctl=GetDlgItem(hwnd,IDC_PMODE);
				SendMessage(hctl,CB_RESETCONTENT,0,0);
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"Normal");
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"Frame-Transfer");
				SendMessage(hctl,CB_SETCURSEL,(WPARAM)params[10],0);
				hctl=GetDlgItem(hwnd,IDC_SHTR_MODE);
				SendMessage(hctl,CB_RESETCONTENT,0,0);
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"Never");
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"preexposure");
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"presequence");
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"pretrigger");
				SendMessage(hctl,CB_ADDSTRING,0,(LPARAM)"no change");
				SendMessage(hctl,CB_SETCURSEL,(WPARAM)params[11],0);
				pl_get_param(hCam,PARAM_TEMP,ATTR_CURRENT,(void *)&currtemp);
				fcurrtemp=((float)currtemp)/100.0f;
				sprintf(temptext,"%f",fcurrtemp);
				SetDlgItemText(hwnd,IDC_TEMP,temptext);
			}
		break;
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDC_CAM_OK:
				{
					//here just carry on to the same message as the enter key processes
				}
				case IDOK:
				{
					int i;
					params[6]=GetDlgItemInt(hwnd,IDC_CLEAR_CYCLES,NULL,FALSE);
					params[7]=(int)SendMessage(GetDlgItem(hwnd,IDC_CLEAR_MODE),CB_GETCURSEL,0,0);
					params[8]=1+(int)SendMessage(GetDlgItem(hwnd,IDC_GAIN_INDEX),CB_GETCURSEL,0,0);
					params[9]=GetDlgItemInt(hwnd,IDC_GAIN_MULT,NULL,FALSE);
					if(params[9]>4095){params[9]=4095;}
					params[10]=(int)SendMessage(GetDlgItem(hwnd,IDC_PMODE),CB_GETCURSEL,0,0);
					params[11]=(int)SendMessage(GetDlgItem(hwnd,IDC_SHTR_MODE),CB_GETCURSEL,0,0);
		    	i=set_params();
			    if(i!=0){
			    	pl_error_message(i,dumstr);
			    	MessageBox(hdlgwnd,dumstr,0,MB_OK);
			   		SendMessage(hdlgwnd,WM_CLOSE,0,0);
					}
					EndDialog(hwnd,0);
				}
				break;
				case IDC_CAM_CANCEL:
				{
					EndDialog(hwnd,0);
				}
			}
		break;
		case WM_CLOSE:

			EndDialog(hwnd, 0);
		break;
		default:
			return FALSE;
	}
	return TRUE;
}

int round(float x)
{
	return (int)((double)x+0.5);
}

int SetDlgItemFlt(float x, HWND hWnd, int ITEM_ID)
{
	char ptext[20];
	sprintf(ptext,"%f",((double)x));
	SetDlgItemText(hWnd,ITEM_ID,ptext);
	return 0;
}

float GetDlgItemFlt(HWND hWnd, int ITEM_ID)
{
	char ptext[20];
	float fltval;
	GetDlgItemText(hWnd,ITEM_ID,ptext,6);
	fltval=(float)atof(ptext);
	return fltval;
}

int set_params()
{
  uns16 clear_cycles,gain_index,gain_multiplier,exp_res_index;
  uns32 clear_mode,pmode,shutter_mode;
  rs_bool cam_status;
  int16 error;
  error=0;
  clear_cycles=params[6]; clear_mode=params[7]; gain_index=params[8];
  gain_multiplier=params[9]; pmode=params[10]; shutter_mode=params[11];
  cam_status=pl_set_param(hCam,PARAM_CLEAR_CYCLES,(void *)&clear_cycles);
  if(cam_status){}
  else{
  	return error;
  }
  pl_set_param(hCam,PARAM_CLEAR_MODE,(void *)&clear_mode);
  pl_set_param(hCam,PARAM_GAIN_INDEX,(void *)&gain_index);
  pl_set_param(hCam,PARAM_GAIN_MULT_FACTOR,(void *)&gain_multiplier);
  pl_set_param(hCam,PARAM_PMODE,(void *)&pmode);
  pl_set_param(hCam,PARAM_SHTR_OPEN_MODE,(void *)&shutter_mode);
  exp_res_index=1;
  pl_set_param(hCam,PARAM_EXP_RES_INDEX,(void *)&exp_res_index);
	return 0;
}

void AcquireContinuous(void* ch)
{
		FILE *fpout,*fpjrn;
    rgn_type region;
    uns32 buffer_size, frame_size;
    uns16 *buffer;
    int16 status;
    uns32 not_needed;
    void_ptr address;
    uns16 numberframes;
    uns32 exposure_time;
    rs_bool cam_status;
    int16 error;
    char msg[100];
    struct _timeb starttime;
    struct _timeb endtime;
    double tottime;
    double frame_time;
    int frames,counter,i,divider,pauses,frameswritten;
    bool stopped,pausefinished;
    
    //set up the parameters
    region.s1=params[0]; region.s2=params[0]+params[1]-1; region.sbin=params[2];
    region.p1=params[3]; region.p2=params[3]+params[4]-1; region.pbin=params[5];
    exposure_time=params[12];
    
    if(exposure_time>=100000){divider=1;}
    else{divider=10;}
    
    tottime=0.0;
    
    //Init a sequence set the region, exposure mode and exposure time
    pl_exp_init_seq();
    pl_exp_setup_cont( hCam, 1, &region, TIMED_MODE, exposure_time, &frame_size, 
                       CIRC_NO_OVERWRITE );

    //set up a circular buffer of 20 frames
    buffer_size = frame_size * 20;
    buffer = (uns16*)malloc( buffer_size );
   
    i=(int)strlen(szFileName);
    memset(dumstr,0,sizeof(dumstr));
    strncpy(dumstr,szFileName,i-3);
		strcat(dumstr,"jrn");
    fpout = fopen(szFileName,"wb");
    fpjrn = fopen(dumstr,"w");
    
    fprintf(fpjrn,"roi serial start %d\n",params[0]);
    fprintf(fpjrn,"roi serial width %d\n",params[1]);
    fprintf(fpjrn,"roi serial bin %d\n",params[2]);
    fprintf(fpjrn,"roi parrallel start %d\n",params[3]);
    fprintf(fpjrn,"roi parrallel width %d\n",params[4]);
    fprintf(fpjrn,"roi parrallel bin %d\n",params[5]);
    fprintf(fpjrn,"clear cycles %d\n",params[6]);
    fprintf(fpjrn,"clearing mode (0-never,1-preexposure,2-presequence...) %d\n",params[7]);
    fprintf(fpjrn,"amplifier gain %d\n",params[8]);
    fprintf(fpjrn,"on-chip multiplier gain (0-4095) %d\n",params[9]);
    fprintf(fpjrn,"pmode(0-normal,1-frame-transfer,...) %d\n",params[10]);
    fprintf(fpjrn,"shutter open mode(0-never,1-preexposure,2-presequence,3-pretrigger,4-no change) %d\n",params[11]);
    fprintf(fpjrn,"exposure time (microseconds) %d\n",params[12]);
    
    stopped=false;
    pauses=0;
    pausefinished=true;
    frameswritten=0;
pstart:
		if(pausefinished){
			running=true;
    	//Init a sequence set the region, exposure mode and exposure time
   	 	pl_exp_init_seq();
    	pl_exp_setup_cont( hCam, 1, &region, TIMED_MODE, exposure_time, &frame_size, 
                       CIRC_NO_OVERWRITE );

    	//set up a circular buffer of 20 frames
    	buffer_size = frame_size * 20;
    	buffer = (uns16*)malloc( buffer_size );
    	pausefinished=false;
    }

    //start the image display thread
		_beginthread(update_acquire_image,0,NULL);

    //Start the acquisition
    pl_exp_start_cont(hCam, buffer, buffer_size );

    //get the time
		_ftime(&starttime);
    
    //ACQUISITION LOOP
    frames=0;
    counter=1;
    while(running) {
        //wait for data or error
        while( pl_exp_check_cont_status( hCam, &status, &not_needed, 
                                         &not_needed ) && 
               (status != READOUT_COMPLETE && status != READOUT_FAILED) );

        //Check Error Codes
        if( status == READOUT_FAILED ) {
            goto line1;
        }

        if ( pl_exp_get_oldest_frame( hCam, &address )) {
            //address now points to valid data
            if(writefile){
            	fwrite(address,sizeof(uns16),frame_size/sizeof(uns16),fpout);
            	frameswritten++;
            }
            frames++;
            if(frames>divider*counter){
            	SetDlgItemInt(hdlgwnd,IDC_NFRAMES,frames,FALSE);
							memcpy((void*)prevdata2,address,frame_size);
							counter++;
						}
            pl_exp_unlock_oldest_frame( hCam );
        }
        if(paused_acq && (frames==pframes)){running=false; pauses++; paused=true;}
    }
    
    //get the time again
    _ftime(&endtime);
    tottime += ((double)endtime.time+(double)endtime.millitm/1000.0)-((double)starttime.time+(double)starttime.millitm/1000.0);
 
    if(paused_acq && paused){
    	pl_exp_stop_cont(hCam,CCS_HALT);
    	pl_exp_finish_seq( hCam, buffer, 0);
    	pl_exp_uninit_seq();
    	free( buffer );
    	counter=0;
    	while(paused && counter<psec){
    		Sleep(1000L);
    		counter++;
    	}
			//MessageBox(hdlgwnd,"Pause complete","Update",MB_OK);
    	if(counter<psec){
    		stopped=true;
				//MessageBox(hdlgwnd,"Canceled during pause","Update",MB_OK);
    		goto pend;
    	}
    	else{
    		paused=false;
    		pausefinished=true;
    		frames=0;
    		goto pstart;
    	}
    }
    
    pend:
    
    frame_time=tottime/(double)(pauses*pframes+frames);
    fprintf(fpjrn,"frame time = %f seconds\n",(float)frame_time);
    fprintf(fpjrn,"total time = %f seconds\n",(float)tottime);
    fprintf(fpjrn,"frames written = %d\n",frameswritten);
    
    line1:
    
    fclose(fpjrn);
    fclose(fpout);

    //Stop the acquisition
    if(!stopped){
    	pl_exp_stop_cont(hCam,CCS_HALT);
    	//Finish the sequence
    	pl_exp_finish_seq( hCam, buffer, 0);
    	//Uninit the sequence
    	pl_exp_uninit_seq();
    	free( buffer );
    }


    running=false;
    if(status==READOUT_FAILED){
    	MessageBox(hdlgwnd,"Readout Failed",NULL,MB_OK);
    }
    else{
    	MessageBox(hdlgwnd,"Readout Successfully Completed","Update",MB_OK);
    }
}

void PreviewContinuous(void* ch)
{
    rgn_type region;
    uns32 buffer_size, frame_size;
    uns16 *buffer;
    int16 status;
    uns32 not_needed;
    void_ptr address;
    uns16 numberframes;
    uns32 exposure_time;
    rs_bool cam_status;
    int16 error;
    char msg[100];
    
    //set up the parameters
    region.s1=0; region.s2=127; region.sbin=1;
    region.p1=0; region.p2=127; region.pbin=1;
    exposure_time=10000;
    
    //Init a sequence set the region, exposure mode and exposure time
    pl_exp_init_seq();
    pl_exp_setup_cont( hCam, 1, &region, TIMED_MODE, exposure_time, &frame_size, 
                       CIRC_OVERWRITE );

    //set up a circular buffer of 10 frames
    buffer_size = frame_size * 10;
    buffer = (uns16*)malloc( buffer_size );
    ZeroMemory((void*)buffer,buffer_size);

		//start the image drawing thread
		_beginthread(update_preview_image,0,NULL);

    //Start the acquisition
    pl_exp_start_cont(hCam, buffer, buffer_size );
    
    //ACQUISITION LOOP
    while(running) {
        //wait for data or error
        while( pl_exp_check_cont_status( hCam, &status, &not_needed, 
                                         &not_needed ) && 
               (status != READOUT_COMPLETE && status != READOUT_FAILED) );

        //Check Error Codes
        if( status == READOUT_FAILED ) {
            goto line1;
        }

        if ( pl_exp_get_latest_frame( hCam, &address )) {
            //address now points to valid data
						memcpy((void*)prevdata,address,frame_size);
        }
    }

    line1:

    //Stop the acquisition
    pl_exp_stop_cont(hCam,CCS_HALT);

    //Finish the sequence
    pl_exp_finish_seq( hCam, buffer, 0);
    
    //Uninit the sequence
    pl_exp_uninit_seq();

    free( buffer );
    running=false;
    if(status==READOUT_FAILED){
    	MessageBox(hdlgwnd,"Readout Failed",NULL,MB_OK);
    }
}

void update_preview_image(void *dummy)
{
	//this thread draws the image to the screen as long as the aquisition is running
	double dumdouble;
	int dumint,i,j,size;
	HDC hdc;
	size=128;
	hdc=GetDC(hdlgwnd);
	while(running){
		if(autoscale){
			minint=(double)minval(prevdata,size*size);
			maxint=(double)maxval(prevdata,size*size);
			if(maxint<1){maxint=1;}
			SetDlgItemInt(hdlgwnd,IDC_MIN,(int)minint,FALSE);
			SetDlgItemInt(hdlgwnd,IDC_MAX,(int)maxint,FALSE);
		}
		for(i=0;i<size;i++){
			for(j=0;j<size;j++){
				dumdouble=((double)prevdata[j+i*size]-minint)/(maxint-minint);
				dumint=(int)(dumdouble*255.0);
				if(dumint>255){dumint=255;}
				if(dumint<0){dumint=0;}
				SetPixel(prevdc,j,i,RGB((BYTE)dumint,(BYTE)dumint,(BYTE)dumint));
			}
		}
		BitBlt(hdc,0,0,size,size,prevdc,0,0,SRCCOPY);
		SelectObject(hdc,GetStockObject(DC_PEN));
		SetDCPenColor(hdc,RGB(0,0,255));
		MoveToEx(hdc,params[0],params[3],NULL);
		LineTo(hdc,params[0]+params[1]-1,params[3]);
		LineTo(hdc,params[0]+params[1]-1,params[3]+params[4]-1);
		LineTo(hdc,params[0],params[3]+params[4]-1);
		LineTo(hdc,params[0],params[3]);
	}
	ReleaseDC(hdlgwnd,hdc);
}

void update_acquire_image(void *dummy)
{
	//this thread draws the image to the screen as long as the aquisition is running
	double dumdouble;
	int dumint,i,j,sizex,sizey;
	HDC hdc;
	sizex=params[1]/params[2];
	sizey=params[4]/params[5];
	hdc=GetDC(hdlgwnd);
	while(running){
		if(autoscale){
			minaqint=(double)minval(prevdata2,sizex*sizey);
			maxaqint=(double)maxval(prevdata2,sizex*sizey);
			if(maxint<1){maxint=1;}
			SetDlgItemInt(hdlgwnd,IDC_MINAQ,(int)minaqint,FALSE);
			SetDlgItemInt(hdlgwnd,IDC_MAXAQ,(int)maxaqint,FALSE);
		}
		avgaqint=avgval(prevdata2,sizex*sizey);
		SetDlgItemFlt((float)avgaqint,hdlgwnd,IDC_AVGAQ);
		for(i=0;i<sizey;i++){
			for(j=0;j<sizex;j++){
				dumdouble=((double)prevdata2[i*sizex+j]-minaqint)/(maxaqint-minaqint);
				dumint=(int)(dumdouble*255.0);
				if(dumint>255){dumint=255;}
				if(dumint<0){dumint=0;}
				SetPixel(acqdc,j,i,RGB((BYTE)dumint,(BYTE)dumint,(BYTE)dumint));
			}
		}
		StretchBlt(hdc,400,0,sizex*zoom,sizey*zoom,acqdc,0,0,sizex,sizey,SRCCOPY);
		Sleep(100L);
	}
	ReleaseDC(hdlgwnd,hdc);
}

int minval(unsigned short* data,int length)
{
	int i,temp;
	temp=data[0];
	for(i=1;i<length;i++){
		if(data[i]<temp){temp=data[i];}
	}
	return temp;
}

int maxval(unsigned short* data,int length)
{
	int i,temp;
	temp=data[0];
	for(i=1;i<length;i++){
		if(data[i]>temp){temp=data[i];}
	}
	return temp;
}

double avgval(unsigned short* data,int length)
{
	int i;
	double avg;
	avg=0.0;
	for(i=0;i<length;i++){
		avg+=data[i];
	}
	return avg/length;
}